#include <avr/eeprom.h>
#include <avr/pgmspace.h>
#include "global.h"
#include "nv_memory.h"
#include "ui.h"
#include "filters.h"

extern uint8_t nMode;    // FM/AM/AUX
extern uint8_t nRFMode;  // FM/AM

extern uint8_t nVolume;         // Audio volume control, 0-63
extern int8_t nSquelch[];
extern uint8_t nBacklightAdj;   // LCD backlight value, 0-255
extern uint8_t nBacklightKeep;  // LCD backlight auto keep seconds, 0-255, 0 for always on
extern uint8_t nScanStayTime;   // Seconds to stay at current frequency
extern uint8_t nAnyHoldTime;    // Seconds to hold current frequency when lost signal
extern uint8_t AGC;
extern uint8_t nAGC;
extern uint8_t nDeemphasis;
extern uint8_t nTuneType;       // Freq_Step/Seek/Ch/Scan/Any/Scan_Save

extern uint8_t nBand;  // Band: LW/MW/SW/SH/FL/FM/AL/AB/VH
extern const int32_t nBandFMin[9];
extern const int32_t nBandFMax[9];
extern int32_t nBandFreq[9];  // Band default frequency
extern int8_t nBandCh[9];
extern const int8_t nBandChs[9];  // Band total channels

extern uint8_t nStepIdx;     // Step index for current band

extern int8_t nAMFilter;  // Current FIR filter index for AM
extern int8_t nFMFilter;  // Current FIR filter index for FM(ADP_FM_FILTER is adaptive)



void NVMInit(void)
{
#if DEBUG
    Serial.println(F("NVMInit"));
#endif
    LCDXYStr_P(0, 1, (PGM_P)F("Init NV Mem? "));
    if(YesNo(false)) {
        LCDXYStr_P(0, 1, (PGM_P)F("Initializing ..."));
        NVMUnpkWrData(NVM_INIT, NVMADDR_SIG);  // Initialize NV memory
    }
}


void NVMGetArgs(void)
{
    uint8_t u8, i;
    uint16_t u16;

    if(eeprom_read_dword((uint32_t *)NVMADDR_SIG) != (NVMSIG))
        NVMInit();

    nBand = NV_read_byte((uint8_t *)NVMADDR_BAND);
    if(nBand >= NUM_BANDS)
        nBand = BAND_FM;

    nVolume = NV_read_byte((uint8_t *)NVMADDR_VOL);
    if(nVolume > MAX_VOL)
        nVolume = 30;

    nMode = NV_read_byte((uint8_t *)NVMADDR_MODE);
    if(nMode >= NUM_MODES)
        nMode = MODE_FM;

    nRFMode = NV_read_byte((uint8_t *)NVMADDR_RFMODE);
    if(nRFMode > MODE_FM)
        nRFMode = MODE_FM;

    nTuneType = NV_read_byte((uint8_t *)NVMADDR_TUNE);
    if(nTuneType >= NUM_TYPES)
        nTuneType = TYPE_FREQ;

    nStepIdx = NV_read_byte((uint8_t *)NVMADDR_STEPIDX);
    if(nStepIdx >= NUM_STEPS)
        nStepIdx = 0;

    nAMFilter = NV_read_byte((uint8_t *)NVMADDR_AMFILTER);
    if(nAMFilter >= NUM_FILTERS)
        nAMFilter = DEF_AM_FILTER;

    nFMFilter = NV_read_byte((uint8_t *)NVMADDR_FMFILTER);
    if(nFMFilter > NUM_FILTERS)
        nFMFilter = DEF_FM_FILTER;

    u8 = NV_read_byte((uint8_t *)NVMADDR_MISC1);
    if(u8 & B00100000)
        AGC |= B00000001;  // RF +6dB
    nAGC = u8 & 3;  // AGC threshold
    nDeemphasis = u8 >> 6;
    if(nDeemphasis > 2)
        nDeemphasis = DEEMPH50;

    nSquelch[0] = NV_read_byte((uint8_t *)NVMADDR_SQU1);
    nSquelch[0] = constrain(nSquelch[0], -99, 99);
    nSquelch[1] = NV_read_byte((uint8_t *)NVMADDR_SQU2);
    nSquelch[1] = constrain(nSquelch[1], -99, 99);
    nScanStayTime = NV_read_byte((uint8_t *)NVMADDR_SCSTAY);
    nAnyHoldTime = NV_read_byte((uint8_t *)NVMADDR_ANYHOLD);
    nBacklightAdj = NV_read_byte((uint8_t *)NVMADDR_BKADJ);
    nBacklightKeep = NV_read_byte((uint8_t *)NVMADDR_BKKEEP);

    for(i = 0; i < NUM_BANDS; i++) {
        nBandCh[i] = NV_read_byte((uint8_t *)NVMADDR_BANDCH + i);
        if(nBandCh[i] >= nBandChs[i])
            nBandCh[i] = 0;

        u16 = NV_read_word((uint16_t *)NVMADDR_BANDFREQ + i);
        if(u16 == 0xffff)  // Blank ch
            nBandFreq[i] = nBandFMin[i];
        else if(i <= BAND_SW)
            nBandFreq[i] = nBandFMin[i] + u16;
        else
            nBandFreq[i] = nBandFMin[i] + (int32_t)u16 * 5;
        nBandFreq[i] = constrain(nBandFreq[i], nBandFMin[i], nBandFMax[i]);
    }
}


void NVMUnpkWrData(const uint8_t* pPGMAddr, uint8_t *pNVMAddr)
{
    uint8_t *pF = (uint8_t *)pPGMAddr;
    uint8_t *pNVM = pNVMAddr;
    uint8_t len, i, val;

    for(;;) {
        len = pgm_read_byte_near(pF++);
        if(!len)
            break;
        if(len & 0x80) {  // Dup
            val = pgm_read_byte_near(pF++);
            for(i = 0; i < (len & 0x7f); i++)
                NV_write_byte(pNVM++, val);
        }
        else {  // len bytes data
            for(i = 0; i < len; i++)
                NV_write_byte(pNVM++, pgm_read_byte_near(pF++));
        }
    }
}


#if DEBUG
uint8_t NV_read_byte(uint8_t *p)
{
    uint8_t val;

    val = eeprom_read_byte(p);
    Serial.print(F("NVRd8 a="));
    Serial.print((uint16_t)p);
    Serial.print(F(" d="));
    Serial.println(val);
    return val;
}


uint16_t NV_read_word(uint16_t *p)
{
    uint16_t val;

    val = eeprom_read_word(p);
    Serial.print(F("NVRd16 a="));
    Serial.print((uint16_t)p);
    Serial.print(F(" d="));
    Serial.println(val);
    return val;
}


void NV_write_byte(uint8_t *p, uint8_t val)
{
    Serial.print(F("NVWr8 a="));
    Serial.print((uint16_t)p);
    Serial.print(F(" d="));
    Serial.println(val);
    eeprom_write_byte(p, val);
}


void NV_write_word(uint16_t *p, uint16_t val)
{
    Serial.print(F("NVWr16 a="));
    Serial.print((uint16_t)p);
    Serial.print(F(" d="));
    Serial.println(val);
    eeprom_write_word(p, val);
}
#endif
